==> rview-1.2.0.tgz: package/lib/app.js <==
/**
 * Create an app
 * @function app
 * @returns {Koa} - Koa app
 */
'use strict'

const { AKoa } = require('akoa')
const co = require('co')
const assert = require('assert')
const { spinalcase } = require('stringcase')
const aslogger = require('aslogger')
const api = require('./api')
const { isProduction } = require('asenv')
const { exceptionMiddleware, staticMiddleware } = require('akoa/middlewares')
const { DEFAULT_URL } = require('../constants')

/** @lends app */
function app (modules) {
  assert.ok(modules, 'modules is required')

  let logger = aslogger({
    disabled: isProduction()
  })

  let koa = AKoa.newApp([])
  let router = AKoa.newRouter()
  modules = [].concat(modules || [])
  modules.forEach((modules) => {
    let { $pathname = DEFAULT_URL } = modules

    if ($pathname && !/^\//.test($pathname)) {
      throw new Error(`[rview] $pathname should start with "/" (given: ${$pathname})`)
    }

    // Register build-in middlewares
    {
      let { $statics = [ 'public' ] } = modules
      let prefix = $pathname
      let middlewares = [
        exceptionMiddleware({ prefix }),
        staticMiddleware($statics, { prefix })
      ]
      for (let middleware of middlewares) {
        koa.use(middleware)
      }
    }

    // Register custom middlewares
    {
      let { $serverMiddlewares = [] } = modules
      for (let middleware of $serverMiddlewares) {
        koa.use(middleware)
      }
    }

    // Register custom middlewares
    {
      let { $middlewares = [] } = modules
      for (let middleware of $middlewares) {
        router.use($pathname, middleware)
      }
    }

    // Register view routes
    {
      let moduleNames = Object.keys(modules).filter((name) => !/^\$/.test(name))
      let {
        $before = null,
        $after = null,
        $compile,
        $urls = {}
      } = modules
      let moduleRootRouter = AKoa.newRouter()
      moduleRootRouter.head('/', co.wrap(function * knock (ctx) {
        ctx.body = null
      }))
      for (let moduleName of moduleNames) {
        logger.debug(`[rview] Register urls to "${moduleName}" module: `)
        let moduleApi = api(moduleName, modules[ moduleName ], {
          $before, $after, $compile, $urls
        })
        let routes = moduleApi.routes()
        let allowedMethods = moduleApi.allowedMethods()
        let { endpoints } = moduleApi
        moduleRootRouter.use('', routes, allowedMethods)

        for (let url of Object.keys(endpoints)) {
          logger.trace(`  "${$pathname}${url}" -> ${endpoints[ url ]}`)
        }
      }
      router.use($pathname, moduleRootRouter.routes())
    }
  })

  koa.use(router.routes())

  return koa
}

module.exports = app

==> rview-1.3.2.tgz: package/lib/app.js <==
/**
 * Create an app
 * @function app
 * @returns {Koa} - Koa app
 */
'use strict'

const { AKoa } = require('akoa')
const co = require('co')
const assert = require('assert')
const { spinalcase } = require('stringcase')
const aslogger = require('aslogger')
const api = require('./api')
const { isProduction } = require('asenv')
const { exceptionMiddleware, staticMiddleware } = require('akoa/middlewares')
const { DEFAULT_URL } = require('../constants')

/** @lends app */
function app (modules) {
  assert.ok(modules, 'modules is required')

  let logger = aslogger({
    disabled: isProduction()
  })

  let koa = AKoa.newApp([])
  let router = AKoa.newRouter()
  modules = [].concat(modules || [])
  modules.forEach((modules) => {
    let { $pathname = DEFAULT_URL } = modules

    if ($pathname && !/^\//.test($pathname)) {
      throw new Error(`[rview] $pathname should start with "/" (given: ${$pathname})`)
    }

    // Register build-in middlewares
    {
      let { $statics = [ 'public' ] } = modules
      let prefix = $pathname
      let middlewares = [
        exceptionMiddleware({ prefix }),
        staticMiddleware($statics, { prefix })
      ]
      for (let middleware of middlewares) {
        koa.use(middleware)
      }
    }

    // Register custom middlewares
    {
      let { $serverMiddlewares = [] } = modules
      for (let middleware of $serverMiddlewares) {
        koa.use(middleware)
      }
    }

    // Register custom middlewares
    {
      let { $middlewares = [] } = modules
      for (let middleware of $middlewares) {
        router.use($pathname, middleware)
      }
    }

    // Register view routes
    {
      let moduleNames = Object.keys(modules).filter((name) => !/^\$/.test(name))
      let {
        $before = null,
        $after = null,
        $catcher = (err) => Object.assign({}, err),
        $compile,
        $urls = {}
      } = modules
      let moduleRootRouter = AKoa.newRouter()
      moduleRootRouter.head('/', co.wrap(function * knock (ctx) {
        ctx.body = null
      }))
      for (let moduleName of moduleNames) {
        logger.debug(`[rview] Register urls to "${moduleName}" module: `)
        let moduleApi = api(moduleName, modules[ moduleName ], {
          $before, $after, $compile, $urls, $catcher
        })
        let routes = moduleApi.routes()
        let allowedMethods = moduleApi.allowedMethods()
        let { endpoints } = moduleApi
        moduleRootRouter.use('', routes, allowedMethods)

        for (let url of Object.keys(endpoints)) {
          logger.trace(`  "${$pathname}${url}" -> ${endpoints[ url ]}`)
        }
      }
      router.use($pathname, moduleRootRouter.routes())
    }
  })

  koa.use(router.routes())

  return koa
}

module.exports = app
